@@ -4,7 +4,7 @@ import multiprocessing
 import platform
 import subprocess
 import sys
-from typing import Optional, Dict, List, Any
+from typing import Generator, Optional, Dict, List, Any
 
 import _cycles
 import bpy
@@ -12,11 +12,16 @@ import bpy
 from vendor import distro, cpu_cores
 
 
+# NOTE: Keep in the order of preference.
+# This means that if device supports multiple compute backends only
+# the first one will be used. This is because, for example, it is not
+# so interesting to check CUDA performance on OptiX-capable device.
 class DeviceType(enum.Enum):
     CPU: str = 'CPU'
-    OpenCL: str = 'OPENCL'
-    CUDA: str = 'CUDA'
+    HIP: str = 'HIP'
     OptiX: str = 'OPTIX'
+    CUDA: str = 'CUDA'
+    Metal: str = 'METAL'
 
 
 @dc.dataclass
@@ -29,7 +34,11 @@ class Device:
         if self.type == DeviceType.CPU:
             return {'name': self.name, 'type': self.type.value}
         else:
-            return {'name': self.name, 'type': self.type.value, 'is_display': self.is_display}
+            return {
+                'name': self.name,
+                'type': self.type.value,
+                'is_display': self.is_display,
+            }
 
 
 @dc.dataclass
@@ -37,35 +46,68 @@ class ComputeDevice(Device):
     cycles_device: Any
 
 
+def _get_devices_for_type(type: DeviceType) -> Generator[Any, None, None]:
+
+    try:
+        available_devices = _cycles.available_devices(type.value)
+    except ValueError:
+        # Ignore compute device type which is not supported by the current Cycles version.
+        return
+
+    for device in available_devices:
+        # Device is a non-strictly-typed tuple. Element with index 1 is the type
+        # of the device.
+        if device[1] != type.value:
+            continue
+        yield device
+
+
 def _get_all_devices() -> List[Device]:
-    return [
-        Device(
-            name=name.replace(' (Display)', ''),
-            type=DeviceType(type),
-            is_display='(Display)' in name,
-        )
-        for name, type, *_ in (
-            _cycles.available_devices('CPU') +
-            _cycles.available_devices('OPENCL') +
-            _cycles.available_devices('CUDA') +
-            _cycles.available_devices('OPTIX')
-        )
-    ]
+    all_devices = []
+
+    used_devices = []
+
+    for type in DeviceType:
+        for name, *_ in _get_devices_for_type(type):
+            if name in used_devices:
+                continue
+            used_devices.append(name)
+
+            all_devices.append(
+                Device(
+                    name=name.replace(' (Display)', ''),
+                    type=DeviceType(type),
+                    is_display='(Display)' in name,
+                )
+            )
+
+    return all_devices
 
 
 def _get_compute_devices() -> List[ComputeDevice]:
-    return [
-        ComputeDevice(
-            name=d.name.replace(' (Display)', ''),
-            type=device_type,
-            is_display='(Display)' in d.name,
-            cycles_device=d,
-        )
-        for device_type in DeviceType
-        for d in bpy.context.preferences.addons['cycles'].preferences.get_devices_for_type(
-            device_type.value
-        )
-    ]
+    compute_devices = []
+
+    cycles = bpy.context.preferences.addons['cycles']
+    cpref = cycles.preferences
+
+    for type in DeviceType:
+        try:
+            devices = cpref.get_devices_for_type(type.value)
+        except ValueError:
+            # Ignore compute device type which is not supported by the current Cycles version.
+            continue
+
+        for device in devices:
+            compute_devices.append(
+                ComputeDevice(
+                    name=device.name.replace(' (Display)', ''),
+                    type=type,
+                    is_display='(Display)' in device.name,
+                    cycles_device=device,
+                )
+            )
+
+    return compute_devices
 
 
 @dc.dataclass
@@ -89,7 +131,14 @@ def _get_cpu_topology() -> CPUTopology:
     else:
         sockets = int(
             subprocess.check_output(
-                ('wmic', 'computersystem', 'get', 'NumberOfProcessors', '/value'), text=True
+                (
+                    'wmic',
+                    'computersystem',
+                    'get',
+                    'NumberOfProcessors',
+                    '/value',
+                ),
+                text=True,
             )
             .strip()
             .split('=')[1]
